var FlixSlider = function(className) {

  var defaults = {
    hoverButtons: false,
    itemsOnScreen: 4,
    itemsToMove: 4,
    sizingClass: 'container',
    pips: false,
    debug: false,
    prevBtnIconHtml: '<i class="fas fa-chevron-left"></i>',
    nextBtnIconHtml: '<i class="fas fa-chevron-right"></i>',
    dragOnDesktop: true,
    responsive: [{
        breakpoint: 1200,
        settings: {
          itemsOnScreen: 3,
          itemsToMove: 3
        }
      },
      {
        breakpoint: 992,
        settings: {
          itemsOnScreen: 3,
          itemsToMove: 3
        }
      },
      {
        breakpoint: 768,
        settings: {
          itemsOnScreen: 2,
          itemsToMove: 2
        }
      },
      {
        breakpoint: 576,
        settings: {
          itemsOnScreen: 2,
          itemsToMove: 2
        }
      },
    ]
  }

  let itemsOnScreen = defaults.itemsOnScreen,
    fsItemGutter = 15,
    dragItemsToMove = defaults.itemsOnScreen,
    itemsToMove = defaults.itemsToMove,

    fsArticle = $(className),
    fsTrack,
    trackIndent,
    numberOfPips,
    clickPageCounter = 0,
    currentPage = 1,
    fsItem,
    containerWidth,
    screenWidth = $(document).width(),

    fsItemWidth,
    fsItemNumber,

    count = 0,
    currentTrackPosition = 0,
    fsButtonPrevIconHTML = defaults.prevBtnIconHtml,
    fsButtonNextIconHTML = defaults.nextBtnIconHtml,
    fsButtonPrev,
    fsButtonNext,
    fsButtonPrevHover,
    fsButtonNextHover,

    dragging = false,
    xDirection = "",
    oldPageX = 0,
    pageXOffset = 50,
    isMobile = false;

  //line up initial track posiion with container
  //
  var setItemWidth = function(items) {
    fsItemWidth = (containerWidth / items) - fsItemGutter * 2

    $(fsItem)
      .css('width', fsItemWidth + 'px')
      .css('flex', '0 0 ' + fsItemWidth + 'px')
      .css('margin-left', fsItemGutter + 'px')
      .css('margin-right', fsItemGutter + 'px')
  }

  // Buttons
  //
  var setUpButtons = function() {
    if (fsItemNumber > defaults.itemsOnScreen) {

      setUpButtonIcon()

      // set up button vars
      //
      fsButtonPrev = fsArticle.find('.fs-prev')
      fsButtonNext = fsArticle.find('.fs-next')

      calcButtonWidth()

      if (defaults.hoverButtons == true) {
        // Button hovers
        //
        $(fsButtonPrev).on('mouseenter', function() {
          fsButtonPrevHover = setTimeout(function callPrev() {
            prevItem(itemsToMove)
            fsButtonPrevHover = setTimeout(callPrev, 1000) // (*)
          }, 100)
        })

        $(fsButtonPrev).on('mouseleave', function() {
          clearTimeout(fsButtonPrevHover)
        })

        $(fsButtonNext).on('mouseenter', function() {
          fsButtonNextHover = setTimeout(function callNext() {
            nextItem(itemsToMove)
            fsButtonNextHover = setTimeout(callNext, 1000) // (*)
          }, 100)
        })

        $(fsButtonNext).on('mouseleave', function() {
          clearTimeout(fsButtonNextHover)
        })
      } else if (defaults.hoverButtons == false) {
        // Button click
        //
        $(fsButtonPrev).on('click', function() {
          prevItem(itemsToMove)
        })
        $(fsButtonNext).on('click', function() {
          nextItem(itemsToMove)
        })
      }
    }
  }

  var setUpButtonIcon = function () {
    if(fsButtonPrevIconHTML != null || fsButtonNextIconHTML != null) {
      $(fsArticle)
        .append('<button class="fs-prev" tabindex="-1">' + fsButtonPrevIconHTML + '<span class="sr-only">Previous</span></button>')
        .append('<button class="fs-next" tabindex="-1">' + fsButtonNextIconHTML + '<span class="sr-only">Next</span></button>')
    } else {
      $(fsArticle)
        .append('<button class="fs-prev" tabindex="-1"><span class="fs-prev-icon"></span><span class="sr-only">Previous</span></button>')
        .append('<button class="fs-next" tabindex="-1"><span class="fs-next-icon"></span><span class="sr-only">Next</span></button>')
    }
    return
  }

  // create the track div
  //
  var setUpTrack = function() {
    $(fsArticle).wrapInner('<div class="fs-track"></div>')
    fsTrack = fsArticle.find('.fs-track')

    if (defaults.debug == true) {
      fsTrack.css('background-color', 'rgba(0,0,255,0.2)')
      fsArticle.append('<div class="fs-touch-event-indicator"></div>')
    }
  }

  // set button width to space either side of sizing div
  //
  var calcButtonWidth = function() {
    fsButtonPrev.css({
      'width': trackIndent - fsItemGutter,
      'opacity': '0'
    })
    fsButtonNext.css('width', trackIndent - fsItemGutter)
  }

  // calc number of items and set track width container
  //
  var calcTrackWidth = function() {
    $(fsTrack).css('width', (fsItemWidth + fsItemGutter * 2) * fsItemNumber + 'px')
  }

  // set the sizing div to posision track to
  //
  var setUpSizingDiv = function() {
    $(fsArticle).append('<div class="' + defaults.sizingClass + ' fs-sizer" tabindex="-1"></div> ')
    containerWidth = $('.fs-sizer').innerWidth()

    if (defaults.debug == true) {
      $('.fs-sizer').css({ 'height': '5px', 'background-color': 'red' })
    }
  }

  var setUpItems = function() {
    $(fsArticle).addClass('flix-slider')
    $(fsArticle).children().wrap('<div class="fs-item"></div>')
    fsItem = fsArticle.find('.fs-item')
    fsItemNumber = $(fsItem).length
  }

  // set starting point of first item and align with container
  //
  var calcTrackIndent = function() {
    trackIndent = (screenWidth - containerWidth) / 2
    $(fsTrack).css('left', trackIndent)
  }

  // prev button function
  //
  var prevItem = function(amountToMove) {
    moveTrack('prev', amountToMove)
  }

  // next button function
  //
  var nextItem = function(amountToMove) {
    moveTrack('next', amountToMove)
  }

  var setArrowKeys = function () {
    $(fsArticle).keydown(function(e) {
      switch (e.which) {
        case 9: // tab
        if(e.shiftKey) {
          prevItem(1) // shift tab
          break
        }
        nextItem(1)
        break

        case 37: // left
        prevItem(itemsToMove)
          break

        case 39: // right
        nextItem(itemsToMove)
          break

        default:
          return
      }
      // e.preventDefault()
    })
  }

  // move track left or right
  //
  var moveTrack = function(direction, amountToMove) {
    if (direction === 'next') {
      if (count + itemsOnScreen < fsItemNumber) { // stop items scrolling past right of container
        // calc if track can advance that far
        //
        if (count + itemsOnScreen + amountToMove > fsItemNumber) {
          var newAmountToMove = fsItemNumber - (count + itemsOnScreen)
          advanceTrack(newAmountToMove)

        } else {
          advanceTrack(amountToMove)
        }
      }
    } else {
      if (count != 0) {
        // calc if track can decrease that far
        //
        if (count - amountToMove < 0) {
          var newAmountToMove = Math.abs(0 - count)
          return decreaseTrack(newAmountToMove)
        }

        return decreaseTrack(amountToMove)
      }
    }

    if (defaults.debug) {
      console.log('at position:' + count, 'itemsOnScreen:' + itemsOnScreen, 'amountToMove:' + amountToMove, 'totalItems:' + fsItemNumber)
    }
  }

  var advanceTrack = function(amountToMoveVal) {
    count = count + amountToMoveVal
    currentTrackPosition = parseInt(currentTrackPosition + (fsItemWidth + fsItemGutter * 2) * amountToMoveVal)
    $(fsTrack).css('transform', 'translateX( -' + Math.abs(currentTrackPosition) + 'px )')
    hideButtons()
    if (defaults.pips === true && itemsToMove >= 4) {
      modifyClickCount('forward')
    }
    // pipActiveState()
  }

  var decreaseTrack = function(amountToMoveVal) {
    count = count - amountToMoveVal
    currentTrackPosition = parseInt(currentTrackPosition - (fsItemWidth + fsItemGutter * 2) * amountToMoveVal)
    $(fsTrack).css('transform', 'translateX( -' + Math.abs(currentTrackPosition) + 'px )')
    hideButtons()
    if (defaults.pips === true && itemsToMove >= 4) {
      modifyClickCount('back')
    }
    // pipActiveState()
  }

  var modifyClickCount = function(direction) {
    if (direction === 'forward') {
      if (itemsToMove === itemsOnScreen) {
        currentPage++
      }
    }

    if (direction === 'back') {
      currentPage--
    }

    updatePips(currentPage)
  }

  var updatePips = function(currentPage) {
    $('.fs-pip', $(fsArticle)).removeClass('active')

    $.each($('.fs-pip', $(fsArticle)), function(index, pip) {
      if (index + 1 === currentPage) {
        $(pip).addClass('active')
      }
    })
  }

  // if at the start / end of the track hide the relevant button
  //
  var hideButtons = function() {
    if (count + itemsOnScreen == fsItemNumber) {
      $(fsButtonNext).css('opacity', '0')
    } else {
      $(fsButtonNext).css('opacity', '1')
    }

    if (count < 1) {
      $(fsButtonPrev).css('opacity', '0')
    } else {
      $(fsButtonPrev).css('opacity', '1')
    }
  }

  // recalculate track postion used on rsponisve
  //
  var recalcTrackPosition = function() {
    $(fsTrack).css('transform', 'translateX(' + '-' + (fsItemWidth + fsItemGutter * 2) * count + 'px )')
    currentTrackPosition = parseInt((fsItemWidth + fsItemGutter * 2) * count)
  }

  // responsive
  //
  var calcResponsive = function() {
    $(defaults.responsive).each(function(index) {
      if (screenWidth < defaults.responsive[index].breakpoint) {
        itemsOnScreen = defaults.responsive[index].settings.itemsOnScreen
        dragItemsToMove = defaults.responsive[index].settings.itemsOnScreen
        setItemWidth(defaults.responsive[index].settings.itemsOnScreen)
        calcTrackWidth()
        recalcTrackPosition()
        calcButtonWidth()
        itemsToMove = defaults.responsive[index].settings.itemsToMove

      } else if (screenWidth >= defaults.responsive[0].breakpoint) {
        itemsOnScreen = defaults.itemsOnScreen
        dragItemsToMove = defaults.itemsOnScreen
        setItemWidth(itemsOnScreen)
        calcButtonWidth()
        itemsToMove = defaults.itemsToMove

        recalcTrackPosition()
      }
    })
  }

  // drag
  //
  var drag = function() {

    var limit = true
    var touchStartX = 0
    var touchStartY = 0
    var oldTouchX = 0
    var oldTouchY = 0

    // mobile drag events
    //
    $(fsTrack).bind('touchstart', function(e) {

      touchStartX = e.originalEvent.touches[0].clientX
      touchStartY = e.originalEvent.touches[0].clientY
      dragStarted(e)
    })

    $(fsTrack).bind('touchmove', function(e) {
      oldTouchX = e.originalEvent.changedTouches[0].clientX
      oldTouchY = e.originalEvent.changedTouches[0].clientY

      if (touchStartX > oldTouchX) {
        if (limit) {
          nextItem(dragItemsToMove)
          limit = false
        }
      } else if (touchStartX < oldTouchX) {
        if (limit) {
          prevItem(dragItemsToMove)
          limit = false
        }
      }
      if (defaults.debug == true) {
        $('.fs-touch-event-indicator').css('left', oldTouchX)
      }
    })

    $(document).bind('touchend', function(e) {
      dragStopped(e)
    })

    // desktop drag events
    //
    if (defaults.dragOnDesktop === true) {
      $(fsTrack).mousedown(function(e) {
        dragStarted(e)
      })
      $(document).on('mousemove', function(e) {
        dragMove(e)
      })
      $(document).mouseup(function(e) {
        dragStopped(e)
      })

      var dragMove = function(e) {

        if (dragging) {
          if (oldPageX < e.pageX) {
            xDirection = "next drag"
            if (limit) {
              $(fsTrack).addClass('fs-being-dragged') // disable items from being clicked
              nextItem(dragItemsToMove)
              limit = false
            }
            oldPageX = e.pageX - pageXOffset
          } else {
            xDirection = "prev drag"
            if (limit) {
              $(fsTrack).addClass('fs-being-dragged') // disable items from being clicked
              prevItem(dragItemsToMove)
              limit = false
            }
            oldPageX = e.pageX + pageXOffset
          }

          // Set last X coridnate and offset to avoid triggering the wrong direction
          //
          if (defaults.debug == true) {
            $('.fs-touch-event-indicator').css('left', oldPageX)
          }
          return false
        }
      }
    }

    var dragStarted = function(e) {
      dragging = true
      return false
    }

    var dragStopped = function(e) {
      $(fsTrack).removeClass('fs-being-dragged')
      dragging = false
      limit = true
      e.cancelBubble = true
    }
  }

  // init
  //
  setUpItems()
  setUpTrack()
  setUpSizingDiv()
  setItemWidth(itemsOnScreen)
  calcTrackWidth()
  calcTrackIndent()
  setUpButtons()
  drag()
  calcResponsive()
  setArrowKeys()

  // recalulate the starting posion of first item and align with container
  //
  $(window).resize(function() {

    containerWidth = $('.fs-sizer').innerWidth()
    screenWidth = $(document).width()
    calcTrackWidth()
    calcTrackIndent()
    calcResponsive()
  })
}
